from pymop.problems.ackley import *
from pymop.problems.bnh import *
from pymop.problems.cantilevered_beam import *
from pymop.problems.carside import *
from pymop.problems.dtlz import *
from pymop.problems.cdtlz import *
from pymop.problems.ctp import *
from pymop.problems.griewank import *
from pymop.problems.knapsack import *
from pymop.problems.kursawe import *
from pymop.problems.osy import *
from pymop.problems.pressure_vessel import *
from pymop.problems.rastrigin import *
from pymop.problems.rosenbrock import *
from pymop.problems.schwefel import *
from pymop.problems.sphere import *
from pymop.problems.tnk import *
from pymop.problems.truss2d import *
from pymop.problems.welded_beam import *
from pymop.problems.zakharov import *
from pymop.problems.zdt import *
from pymop.problems.g import *
